#include <stdio.h>
#import <Cocoa/Cocoa.h>

#import "Move.h"
#import "Utils.h"
#import "MailDate.h"

@implementation OME_Move
FILE *logFile = 0;

- (int) numberOfRowsInTableView: (NSTableView *)tableView
{
    if ([tableView isEqualTo: moveListTable])
        return [moveList count];
    else
        return 0;
}


- (id) tableView: (NSTableView *)tableView
		objectValueForTableColumn: (NSTableColumn *) tableColumn
		row: (int) row
{
    if ([tableView isEqualTo: moveListTable])
        return [self moveListTableColumn: tableColumn row: row];
    else
        return 0;
}

- (id) moveListTableColumn: (NSTableColumn *)tableColumn
		row: (int)row
{
NSDictionary *dict;
    dict = [moveList objectAtIndex: row];
    switch ([(NSNumber *)[dict objectForKey: @"key"] intValue])
    {
    case FileName:
    case From:
    case Subject:
    case To:
        return [dict objectForKey: @"condition"];
        break;
    case Date:
        return  [dict objectForKey: @"date"];
        break;
    }
    return 0;
}


- (void) awakeFromNib
{
NSCalendarDate *date = [NSCalendarDate calendarDate];

    moveList = [[NSMutableArray alloc] init];
    [moveList addObjectsFromArray: [[NSUserDefaults standardUserDefaults]
            arrayForKey: @"moveList"]];
    [moveListTable reloadData];

	execTime = 3;
	[countDownField setIntValue: execTime];
	execTimer = [NSTimer scheduledTimerWithTimeInterval: 1
						target: self selector: @selector(countDown:)
						userInfo: nil repeats: YES];
    timerIsValid = YES;
    
    conditionY = [conditionField frame].origin.y;
    compareY = [comparePopUp frame].origin.y;
    
    [yearField setObjectValue: date];
    [monthField setObjectValue: date];
    [dayField setObjectValue: date];
	
#if 0
	if (![moveList count])
		[settingButton performClick: self];
#endif
}

- (IBAction) doSetting: (NSButton *) sender
{
    [execTimer invalidate];
    timerIsValid = NO;
    [mainWindow close];
    [settingWindow makeKeyAndOrderFront: self];
    [mainWindow display];

    if ([moveList count])
    {
        [moveListTable selectRow: 0 byExtendingSelection: NO];
        [self moveListClick: self];
    }
	else
		[self moveUIByKey:1];
   
    //[comparePopUp setFrameOrigin: NSMakePoint(-300, 5)]; /*to hide*/
}



- (IBAction) addToList: (NSButton *) sender
{
NSDictionary *dict;

    dict = [self setRule];
    [moveList addObject: dict];
    [moveListTable reloadData];
	[[NSUserDefaults standardUserDefaults]
           setObject: moveList forKey: @"moveList"];
	[[NSUserDefaults standardUserDefaults] synchronize];
}


- (IBAction) modifyList: (NSButton *) sender
{
NSDictionary *dict;
int index = [moveListTable selectedRow];

	if (index < 0)
		return;
    dict = [self setRule];
    [moveList replaceObjectAtIndex: index withObject: dict];
    [moveListTable reloadData];
	[[NSUserDefaults standardUserDefaults]
           setObject: moveList forKey: @"moveList"];
	[[NSUserDefaults standardUserDefaults] synchronize];
}


- (NSMutableDictionary *) setRule
{
NSMutableDictionary *dict = [[[NSMutableDictionary alloc] init] autorelease];
NSCalendarDate *date;

//NSLog(@"addToList");
    [dict setObject: [fromFolderField stringValue] forKey: @"fromFolder"];
    [dict setObject: [NSNumber numberWithInt: [[keyPopUp selectedItem] tag]]
                        forKey: @"key"];
    [dict setObject: [conditionField stringValue] forKey: @"condition"];
    [dict setObject: [NSNumber numberWithInt: [[comparePopUp selectedItem] tag]]
                        forKey: @"compare"];
    [dict setObject: [toFolderField stringValue] forKey: @"toFolder"];
    date = [NSCalendarDate dateWithYear: [yearField intValue]
                month: [monthField intValue] day: [dayField intValue]
                hour: 0 minute: 0 second: 0 timeZone: [NSTimeZone defaultTimeZone]];
    [dict setObject: [date descriptionWithCalendarFormat: @"%Y.%m.%d"]
						forKey: @"date"];
    [dict setObject: [NSNumber numberWithInt: [[dateComparePopUp selectedItem] tag]]
                        forKey: @"dateCompare"];
    return dict;
}


- (IBAction) moveListClick: (id) sender
{
unsigned int index = [moveListTable selectedRow];
NSDictionary *dict;
int key;
NSCalendarDate *date;
NSNumber *dateCompareTag;

    if (index < 0)
        return;
    dict = [moveList objectAtIndex: index];
    [fromFolderField setStringValue: [dict objectForKey: @"fromFolder"]];
    key = [(NSNumber *)[dict objectForKey: @"key"] intValue];
    [keyPopUp selectItemAtIndex: [keyPopUp indexOfItemWithTag: key]];
    [conditionField setStringValue: [dict objectForKey: @"condition"]];
    [comparePopUp selectItemAtIndex: [comparePopUp indexOfItemWithTag:
                [(NSNumber *)[dict objectForKey: @"compare"] intValue]]];
    [toFolderField setStringValue: [dict objectForKey: @"toFolder"]];
    if (key == Date)
        date = [NSCalendarDate dateWithString: [dict objectForKey: @"date"]
					calendarFormat: @"%Y.%m.%d"];
    else
        date = [NSCalendarDate calendarDate];
    [yearField setObjectValue: date];
    [monthField setObjectValue: date];
    [dayField setObjectValue: date];
    dateCompareTag = (NSNumber *)[dict objectForKey: @"dateCompare"];
    if (dateCompareTag)
        [dateComparePopUp selectItemAtIndex: [dateComparePopUp indexOfItemWithTag:
                [dateCompareTag intValue]]];
   
    [self moveUIByKey: key];
}


- (IBAction) chooseKey: (NSPopUpButton *) pop
{
//NSLog(@"%d", [[pop selectedItem] tag]);
    [self moveUIByKey: [[pop selectedItem] tag]];
    [settingWindow display];
}

- (void) moveUIByKey: (int) key
{
    switch (key)
    {
    case FileName:
    case From:
    case Subject:
    case To:
	default:
        [self setHeight: conditionY ofView: conditionField];
        [self setHeight: compareY ofView: comparePopUp];
        [self setHeight: -100 ofView: yearField];
        [self setHeight: -100 ofView: yearLabel];
        [self setHeight: -100 ofView: monthField];
        [self setHeight: -100 ofView: monthLabel];
        [self setHeight: -100 ofView: dayField];
        [self setHeight: -100 ofView: dayLabel];
        [self setHeight: -100 ofView: dateComparePopUp];
        break;
    case Date:
        [self setHeight: -100 ofView: conditionField];
        [self setHeight: -100 ofView: comparePopUp];
        [self setHeight: conditionY ofView: yearField];
        [self setHeight: conditionY ofView: yearLabel];
        [self setHeight: conditionY ofView: monthField];
        [self setHeight: conditionY ofView: monthLabel];
        [self setHeight: conditionY ofView: dayField];
        [self setHeight: conditionY ofView: dayLabel];
        [self setHeight: compareY ofView: dateComparePopUp];
        break;
    }
    [settingWindow display];
}


- (void) setHeight: (double) y ofView: (NSView *) view
{
NSPoint origin;

    origin = [view frame].origin;
    origin.y = y;
    [view setFrameOrigin: origin];
}


- (IBAction) removeListItem: (id) sender
{
int index = [moveListTable selectedRow];

    if (index < 0)
        return;

    [moveList removeObjectAtIndex: index];
    [moveListTable deselectAll: self];
    [moveListTable reloadData];
    [[NSUserDefaults standardUserDefaults]
            setObject: moveList forKey: @"moveList"];
}


- (IBAction) setDefaultHost: (NSButton *) sender
{
unsigned int index = [moveListTable selectedRow];

    if (index < 0)
        return;
    
    [moveList insertObject: [moveList objectAtIndex: index] atIndex: 0];
    [moveList removeObjectAtIndex: index + 1];
    [moveListTable reloadData];
    [moveListTable selectRow: 0 byExtendingSelection: NO];
    [[NSUserDefaults standardUserDefaults]
            setObject: moveList forKey: @"moveList"];
}


- (IBAction) moveAndTerminate: (id) sender
{
    if (timerIsValid)
    {
        [execTimer invalidate];
        timerIsValid = NO;
    }
    [self move: self];
    [NSApp terminate: self];
}


- (IBAction) move: (id) sender
{
NSEnumerator *enumerator;
NSDictionary *moveRule;
NSString *fromFolder;
int key;

    if (SetOMEPath())
        return;
//NSLog(@"Move");
    enumerator = [moveList objectEnumerator];
    while (moveRule = [enumerator nextObject])
    {
        fromFolder = OMERelativePath([moveRule objectForKey: @"fromFolder"]);
        switch (key = [[moveRule objectForKey: @"key"] intValue])
        {
        case FileName:
            [self moveByFileNameFrom: fromFolder withRule: moveRule];
            break;
        case From:
        case Subject:
        case To:
        case Date:
            [self moveByStringHeaderOf: key From: fromFolder withRule: moveRule];
            break;
        default:
            NSLog(@"Undefined move key! %d",
                    [[moveRule objectForKey: @"key"] intValue]);
            exit(-1);
            break;
        }
    }
}


- (void) moveByFileNameFrom: (NSString *)fromFolder
                    withRule: (NSDictionary *)moveRule
{
NSFileManager *fileManager = [NSFileManager defaultManager];
NSEnumerator *enumerator;
NSString *fromFile;
BOOL found;

    enumerator = [[fileManager directoryContentsAtPath:
                        fromFolder] objectEnumerator];
    while (fromFile = [enumerator nextObject])
    {
        found = [self compareString: fromFile
            toKeyword: [moveRule objectForKey:@"condition"]
            withRule: [[moveRule objectForKey:@"compare"] intValue]];

        if (found)
        {
//NSLog(fromFile);
            [fileManager movePath: [fromFolder stringByAppendingPathComponent:
                       fromFile] 
                toPath: [OMERelativePath([moveRule objectForKey: @"toFolder"])
                            stringByAppendingPathComponent: fromFile]
                handler: 0];
        }
    }
}


- (void) moveByStringHeaderOf: (int) key From: (NSString *)fromFolder
                    withRule: (NSDictionary *)moveRule
{
NSFileManager *fileManager = [NSFileManager defaultManager];
NSEnumerator *enumerator;
NSString *fromFile;
BOOL found;
NSString *suffix;
NSString *readFile;
NSString *headerValue;

//NSLog(@"move");
//NSLog(fromFolder);
    enumerator = [[fileManager directoryContentsAtPath:
                        fromFolder] objectEnumerator];
    while (fromFile = [enumerator nextObject])
    {
NSLog(fromFile);
        suffix = [fromFile pathExtension];
        
        if ([suffix isEqualToString: @"mpart"])
        {
            readFile = [self attatchedReadFile: fromFile inFolder: fromFolder];
            if (!readFile)
                continue;
            suffix = [readFile pathExtension];
        }
        else
            readFile = [fromFolder stringByAppendingPathComponent: fromFile];
NSLog(readFile);
NSLog(@"a0");
        
        if ([suffix isEqualToString: @"wmail"])
        {
            if ((key == From) || (key == Date))
                continue;
        }
        else if (!([suffix isEqualToString: @"ygm"]
					|| [suffix isEqualToString: @"mail"]
                    || [suffix isEqualToString: @"rply"]
                    || [suffix isEqualToString: @"htm"]
                    || [suffix isEqualToString: @"html"]
					))
            continue;
        
NSLog(@"a1");
        headerValue = [self headerValue: key ofFile: readFile];
NSLog(headerValue);
        if (!headerValue)
            continue;
        
NSLog(@"key = %d", key);
        switch (key)
        {
        case From:
        case Subject:
        case To:
            found = [self compareString: headerValue
                toKeyword: [moveRule objectForKey:@"condition"]
                withRule: [[moveRule objectForKey:@"compare"] intValue]];
            break;
        case Date:
//NSLog(@"Date");
            found = [self compareDateString: headerValue
                withDate: [NSCalendarDate dateWithString: [moveRule objectForKey: @"date"]
								calendarFormat: @"%Y.%m.%d"]
                withRule: [[moveRule objectForKey:@"dateCompare"] intValue]];
            break;
        default:
            found = 0;
        }

        if (found)
        {
            [fileManager movePath: [fromFolder stringByAppendingPathComponent:
                       fromFile] 
                toPath: [OMERelativePath([moveRule objectForKey: @"toFolder"])
                            stringByAppendingPathComponent: fromFile]
                handler: 0];
        }
    }
}


- (int) compareString: (NSString *) string toKeyword: (NSString *) keyword
            withRule: (int) rule
{
int	found = 0;
    
    switch (rule)
    {
    case Contain:
        found = [string rangeOfString: keyword].length;
        break;
    case BeginWith:
        found = [string hasPrefix: keyword];
        break;
    case EndWith:
        found = [string hasSuffix: keyword];
        break;
    case CoincideWith:
        found = [string isEqualToString: keyword];
        break;
    default:
        NSLog(@"Undfined compare rule %d", rule);
        break;
    }
    
    return found;
}

- (int) compareDateString: (NSString *) string withDate: (NSCalendarDate *) keyDate
            withRule: (int) rule
{
int	found = 0;
NSCalendarDate *mailDate;
int mailDay, keyDay;

    mailDate = CString2Date([string cString]);
    mailDay = [mailDate dayOfCommonEra];
    keyDay = [keyDate dayOfCommonEra];
    switch (rule)
    {
    case DateBefore:
//NSLog(@"DateBefore");
        found = (mailDay < keyDay);
        break;
    case DateUntill:
        found = (mailDay <= keyDay);
        break;
    case DateSince:
        found = (mailDay >= keyDay);
        break;
    case DateAfter:
        found = (mailDay > keyDay);
        break;
    case DateAt:
        found = (mailDay == keyDay);
        break;
    default:
        NSLog(@"Undfined date compare rule %d", rule);
        break;
    }
    
    return found;
}


- (NSString *) attatchedReadFile: (NSString *) fromFile inFolder: (NSString *) fromFolder
{
NSString *baseName  = [[fromFolder stringByAppendingPathComponent: fromFile]
						stringByAppendingPathComponent:
                                [fromFile stringByDeletingPathExtension]];
NSString *readFile;
NSString *extentionArray[] = {@"ygm", @"mail", @"rply", @"htm", @"html", 0};
int i;

    for (i = 0; extentionArray[i]; i++)
    {
        readFile = [baseName stringByAppendingPathExtension: extentionArray[i]];
        if ([[NSFileManager defaultManager] fileExistsAtPath: readFile])
            return readFile;
    }
    return 0;
}


- (NSString *) headerValue: (int) key ofFile: (NSString *) readFile
{
char *headerTitle;
FILE *fp;
char oneLine[4096];
char *headerValue;

    if (key == From)
        headerTitle = "From: ";
    else if (key == Subject)
        headerTitle = "Subject: ";
    else if (key == To)
        headerTitle = "To: ";
    else if (key == Date)
        headerTitle = "Date: ";
    else
        return 0;

    fp = fopen([readFile fileSystemRepresentation], "r");
    if (!fp)
        return 0;
    
    while (fgets(oneLine, 4096, fp))
    {
        if (!oneLine[0] || (oneLine[0] == '\n'))
        {
            fclose(fp);
            return 0;
        }
        if (oneLine[strlen(oneLine)-1] == '\n')
            oneLine[strlen(oneLine)-1] = 0;

        if (HeadStrCL(headerTitle, oneLine))
        {
            headerValue = oneLine + strlen(headerTitle);
            fclose(fp);
            return [[[NSString alloc]
                    initWithData: [NSData dataWithBytes: headerValue
                            length: strlen(headerValue)]
                    encoding: NSShiftJISStringEncoding] autorelease];
        }
    }
    
    fclose(fp);
    return 0;
}


- (void) countDown: (NSTimer *) timer
{
    execTime--;
    if (execTime <= 0)
    {
        [timer invalidate];
        timerIsValid = NO;
        [self moveAndTerminate: self];
        return;
    }
	[countDownField setIntValue: execTime];
}


- (IBAction) chooseFromFolder: (NSButton *) sender
{
NSString *folder;

	if (folder = [self chooseFolder])
		[fromFolderField setStringValue: folder];
}


- (IBAction) chooseToFolder: (NSButton *) sender
{
NSString *folder;

	if (folder = [self chooseFolder])
		[toFolderField setStringValue: folder];
}


- (NSString *) chooseFolder
{
NSOpenPanel *openPanel = [NSOpenPanel openPanel];
NSString *chosenFolder;

    SetOMEPath();
    [openPanel setCanChooseDirectories: YES];
    [openPanel setCanChooseFiles: NO];
    [openPanel setAllowsMultipleSelection: NO];
    [openPanel setResolvesAliases: YES];
    if ([openPanel runModalForDirectory: omeRootPathString file: nil types: nil]
            != NSOKButton)
        return 0;
    
    chosenFolder = [openPanel filename];
    if ([chosenFolder hasPrefix: omeRootPathString]
        && (! [chosenFolder isEqualToString: omeRootPathString]))
    {
        chosenFolder = [chosenFolder substringFromIndex: [omeRootPathString length]+1];
    }
    
    return chosenFolder;
}

@end
